const {
  encrypt,
  generateHash,
  checkHash,
} = require("../../config/auth/cryptography");
const { genereteToken } = require("../../config/auth/jwt");
const { decryptDonor } = require("../../util/cryptDonor");
const db = require("../database/db");
const nodemailer = require("nodemailer");
const crypto = require("crypto");
const path = require("path");
const fs = require("fs");

const bcrypt = require('bcrypt');
const { anoMesDia } = require("../../util/dateFormat");
require("dotenv").config({ path: __dirname + "/./../../../.env" });


class usersRepository {
  constructor() {
    this.db = db;
  }




  /**
   * Registra os dados de um usuario
   * @param {*} data dados do usuario
   * @returns mensagem referente ao resultado obtido
   */
  register(data) {

    return new Promise(async (resolve, reject) => {
      // const senha = await generateHash(data.senha);

      this.db.query(
        `INSERT INTO user (razao_social, documento, celular, email, observacao, senha, role) VALUES(?,?,?,?,?,?,?)`,
        [data.razao_social, data.documento, data.celular, data.email, data.observacao, data.senha, data.role],
        async (error, response) => {
          if (error) {
            console.error(error);
            resolve({ error: "deu ruim", code: 203 });
          }
          return resolve({ success: "Usuario cadastrado com sucesso!" });
        }
      );
    });
  }


    /**
     * Realiza o login de um administrador
     * @param {*} data dados do adminstrador
     * @returns mensagem referente ao resultado obtido
     */
    login(data) {
  
      return new Promise(async (resolve, reject) => {
  
        
  
        this.db.query(
          `SELECT * FROM user WHERE email = ? AND role like "ADMIN" `,
          [data.email],
          async (error, response) => {
            if (error) {
              console.error("Database query error: ", error);
              return reject(new Error("Database query error"));
            }
  
            if (response.length === 0) return resolve({ error: `Administrador não encontrado`, code: 203 });
            
  
            const userDB = response[0];  
  
            const isMatch = await bcrypt.compare(data.senha, userDB.senha);

            if (!isMatch) return resolve({ error: "Senha incorreta", code: 401 });
            
  
  
            const auth = crypto.randomBytes(128).toString("HEX");
            userDB.auth = auth;
  
            // Atualiza o banco de dados com o novo auth
            this.db.query(
              "UPDATE user SET auth=? WHERE email=?",
              [auth, data.email],
              async (updateError) => {
                if (updateError) {
                  console.error("Database update error:", updateError);
                  return reject(new Error("Database update error"));
                }
  
                this.db.query(
                  `INSERT INTO logs (id_user, data, acao) VALUES (?,?,?)`,
                  [userDB.id, new Date(), "LOGIN ADMIN"],
                  (error, results) => {
                    if (error) {
                      console.error("Error inserting logs: ", error);
                      return reject(error);
                    }
                    resolve();
                  }
                );
  
                // Gera o token
                try {
                  const token = await genereteToken(userDB);
  
                  // Retorna todos os detalhes de um vendedor junto com o token
                  return resolve({
                    token: token,
                    user: {
                      id: userDB.id,
                      razao_social: userDB.razao_social,
                      documento: userDB.documento,
                      email: userDB.email,
                      celular: userDB.celular,
                      endereco: userDB.endereco,
                      role: userDB.role,
                      auth: auth
                    },
                  });
                } catch (tokenError) {
                  console.error("Token generation error: ", tokenError);
                  return reject(new Error("Token generation error"));
                }
              }
            );
          }
        );
      });
    }

  

  /**
   * Realiza o logout do usuário
   * @param {*} data email do usuário
   * @returns mensagem referente ao resultado obtido
   */
  logout(auth) {
    return new Promise(async (resolve, reject) => {
      this.db.query(
        "UPDATE user set auth=null WHERE auth=?",
        [auth],
        async (error, response) => {
          if (error) return reject(new Error(error));
          return resolve({ success: "Logout realizado com sucesso!" });
        }
      );
    });
  }

  /**
   * Realiza o logout do usuário
   * @param {*} data email do usuário
   * @returns mensagem referente ao resultado obtido
   */
  aprovarUsuario(data) {
    return new Promise(async (resolve, reject) => {
      this.db.query(
        "UPDATE user set status = ? WHERE id = ?",
        [data.status, data.id],
        async (error, response) => {
          if (error) return reject(new Error(error));
          return resolve({ success: "Usuario aprovado com sucesso!" });
        }
      );
    });
  }

  /**
   * Aprova ou recusa o funcionario
   * @param {*} data 
   * @returns mensagem referente ao resultado obtido
   */
    setStatusFunc(data) {
      return new Promise(async (resolve, reject) => {
        this.db.query(
          "UPDATE funcionarios set status = ? WHERE id = ?",
          [data.status, data.id],
          async (error, response) => {
            if (error) return reject(new Error(error));
           return resolve({ success: `Funcionario ${data.status} com sucesso!`, code: 200 });
          }
        );
      });
    }

    /**
   * Aprova um funcionario
   * @param {*} data 
   * @returns mensagem referente ao resultado obtido
   */
  aprovarFuncionario(data) {
    return new Promise(async (resolve, reject) => {
      this.db.query(
        "UPDATE funcionarios set status = ? WHERE id = ?",
        [data.status, data.id],
        async (error, response) => {
          if (error) return reject(new Error(error));
          return resolve({ success: `Funcionario aprovado com sucesso!`, code: 200 });
        }
      );
    });
  }

  /**
   * Recusa um funcionario
   * @param {*} data 
   * @returns mensagem referente ao resultado obtido
   */
  recusarFuncionario(data) {
    return new Promise(async (resolve, reject) => {
      this.db.query(
        "UPDATE funcionarios set status = ? WHERE id = ?",
        [data.status, data.id],
        async (error, response) => {
          if (error) return reject(new Error(error));
          return resolve({ success: `Funcionario recusado com sucesso!`, code: 200 });
        }
      );
    });
  }


  /**
   * Recusa o usuario
   * @param {*} data dados do usuario
   * @returns mensagem referente ao resultado obtido
   */
  RecusarUsuario(data) {
    return new Promise(async (resolve, reject) => {
      this.db.query(
        "UPDATE user set status = ? WHERE id = ?",
        [data.status, data.id],
        async (error, response) => {
          if (error) return reject(new Error(error));
          return resolve({ success: "Usuario recusado com sucesso!" });
        }
      );
    });
  }


  recover(data) {
    return new Promise((resolve, reject) => {
      // Verificar se o email existe no banco de dados
      db.query("SELECT email, senha FROM user WHERE id = ?", [data.id_user], async (err, result) => {
        if (err) return reject("Erro no banco de dados");
        
        const newHashedPassword = await bcrypt.hash(data.senha, 10);

        if (result[0].senha === newHashedPassword) return resolve({ error: "Senha negada!", code: 203 });
        if (data.senha !== data.senhaConfirmacao) return resolve({ error: "Senhas informadas não coincidem", code: 203 });
        

        this.db.query("UPDATE user SET senha = ? WHERE email = ?", [newHashedPassword, result[0].email], (err) => {
          if (err) {
            console.log(err);

            return reject("Erro ao atualizar a senha no banco de dados");
          }

          const templatePath = path.join(__dirname, '../../view/senha-admin.hbs');
          const templateHtml = fs.readFileSync(templatePath, 'utf-8');
    
          const emailHtml = templateHtml.replace('{{senha}}', data.senha)

          let transporter = nodemailer.createTransport({
            host: 'brasapedidos.com.br',
            port: 465,
            secure: true,
            auth: {
              user: process.env.EMAIL_SENDER,
              pass: process.env.EMAIL_PASSWORD,
            },
          });

          let mailOptions = {
            from: 'suporte@brasapedidos.com.br',
            to: data.email,
            subject: 'Alteração de senha',
            html: emailHtml
          };

          transporter.sendMail(mailOptions, (error, info) => {
            if (error) {
              return console.log(error);
            }
            console.log('Email sent: %s', info.messageId);
          });

          return resolve({ success: "Nova senha enviada para o email!" });

        });
      });
    });
  }

  // edit(data, id_user) {
  //   return new Promise(async (resolve, reject) => {
  //     try {
  //       const resultado = data.dias_pedido.join(' ');
  //       const codUltra = Boolean(data.codUltra);
        
  //       let qtde_minima = data.quantidade.map(item => {data.id, item.minimo, item.linha});

  //       let lembrete = data.lembretes.map(item => {data.id_mensagem, item.horario}); 

        
  //       const values = [
  //         data.tabela,
  //         data.linha,
  //         data.id_vendedor,
  //         data.meta_mensal,
  //         resultado,
  //         data.notificacao_email,
  //         data.notificacao_what,
  //         data.aroma,
  //         data.role,
  //         data.lembretes[0].horario
  //       ];
  
  //       if (codUltra) values.push(data.codUltra);
  //       values.push(id_user);
        
  //       await this.db.query(
  //         `UPDATE user SET tipo= ?, linha = ?, id_vendedor = ?, meta_mensal=?, dias_pedido=?, notificacao_email=?, notificacao_what=?, aroma=?, role=?, hora_lembrete=? ${codUltra ? ", codUltra = ?" : ""}  WHERE id = ?`,
  //         values
  //       );
    
  //       for (let item of qtde_minima) {
  //         await this.db.query(
  //           `INSERT INTO quantidade_minima (id_user, quantidade_minima, linha) VALUES (?, ?, ?)`,
  //           item
  //         );
  //         console.log(item.minimo);
          
  //       }

  //       for (let item of lembrete) {
  //         await this.db.query(
  //           `INSERT INTO mensagens_vinculadas (id_mensagem, horario) VALUES (?, ?)`,
  //           item
  //         );
  //       }
        
  //       return resolve({ success: "Usuario atualizado com sucesso!", code: 200 });
  //     } catch (error) {
  //       console.error(error);
  //       return reject(new Error(error));
  //     }
  //   });
  // }

  edit(data, id_user) {
    return new Promise(async (resolve, reject) => {

      if (!id_user) return resolve({ error: "ID do usuario obrigatorio", code: 204 });
      try {

        const resultado = Array.isArray(data.dias_pedidos) ? data.dias_pedidos.join(' ') : '';
        const linhaUser = Array.isArray(data.linhaProdutos) ? data.linhaProdutos.join(',') : '';
        const codUltra = Boolean(data.codUltra);
  
  
        let lembrete = data[0]?.lembretes.map(item => [item.id_mensagem, item.horario]);
        const vendedor = data.vendedor === '' ? null : data.vendedor;

        const values = [
          data.tabela,
          data.aroma,
          data.cargo,
          data.notificacao_whats,
          data.notificacao_email,
          vendedor,
          resultado,
          linhaUser,
          data.meta || null,
          data?.lembretes[0]?.horario ? data?.lembretes[0]?.horario : null
        ];
  
        if (codUltra) values.push(data.codUltra);
        values.push(id_user);
  
        const updateRespose = await new Promise(async (resolve, reject) => {
        this.db.query(
          `UPDATE user SET tipo=?, aroma=?, role=?, notificacao_whats=?, notificacao_email=?, id_vendedor=?, dias_pedido=?, linha=?, meta_mensal=?, hora_lembrete=? ${codUltra ? ", codUltra = ?" : ""}  WHERE id = ?`,
          values
        );
        resolve()
      });





      const registrosAtuais = await new Promise((resolve, reject) => {
        this.db.query(
          `SELECT id, linha, quantidade_minima FROM quantidade_minima WHERE id_user = ?`,
          [id_user],
          (error, results) => {
            if (error) return reject(error);
            resolve(results);
          }
        );
      });
      
      const atuaisMap = new Map();
      for (let item of registrosAtuais) {
        atuaisMap.set(item.linha, {
          id: item.id,
          quantidade_minima: Number(item.quantidade_minima),
        });
      }
      
      const linhasRecebidas = new Set();
      
      for (let item of data.quantidade) {
        const linha = item.linha;
        const minimo = Number(item.minimo);
        linhasRecebidas.add(linha);
      
        const atual = atuaisMap.get(linha);
      
        if (!atual) {
          await new Promise((resolve, reject) => {
            this.db.query(
              `INSERT INTO quantidade_minima (id_user, quantidade_minima, linha) VALUES (?, ?, ?)`,
              [id_user, minimo, linha],
              (error, result) => {
                if (error) return reject(error);
                console.log(`INSERT feito para linha: ${linha}`);
                resolve();
              }
            );
          });
        } else if (atual.quantidade_minima !== minimo) {
          await new Promise((resolve, reject) => {
            this.db.query(
              `UPDATE quantidade_minima SET quantidade_minima = ? WHERE id_user = ? AND linha = ?`,
              [minimo, id_user, linha],
              (error, result) => {
                if (error) return reject(error);
                console.log(`UPDATE para linha: ${linha}`);
                console.log(`Linhas afetadas: ${result.affectedRows}`);
                resolve();
              }
            );
          });

        } else {
          console.log(`Sem mudança para linha: ${linha}`);
        }
      }

      for (let atual of registrosAtuais) {
        if (!linhasRecebidas.has(atual.linha)) {
          await new Promise((resolve, reject) => {
            this.db.query(
              `DELETE FROM quantidade_minima WHERE id_user = ? AND linha = ?`,
              [id_user, atual.linha],
              (error, result) => {
                if (error) return reject(error);
                console.log(`DELETE feito para linha: ${atual.linha}`);
                console.log(`Linhas afetadas: ${result.affectedRows}`);
                resolve();
              }
            );
          });
        }
      }
      
      
      
          



        

          for (let item of data?.lembretes) {
            await this.db.query(
              `INSERT INTO mensagens_vinculadas (id_mensagem, horario, id_user) VALUES (?,?,?)`,
              [item.id_mensagem, item.horario, id_user]
            );
          }
        
  
        return resolve({ success: "Usuario atualizado com sucesso!", code: 200 });
      } catch (error) {
        console.error(error);
        return reject(new Error(error));
      }
    });
  }
  

  listEdit(id_user) {
    return new Promise(async (resolve, reject) => {
        try {
            const user = await new Promise((resolve, reject) => {
                this.db.query(
                    `SELECT user.id, user.aroma, user.role, user.notificacao_email, user.notificacao_whats, user.id_vendedor, user.dias_pedido, 
                    user.meta_mensal AS meta, user.id_vendedor AS vendedor, user.codUltra, user.tipo AS tabela, user.status
                    FROM user
                    WHERE user.id = ?`,
                    [id_user],
                    (error, results) => {
                        if (error) return reject(new Error(error));
                        if (results.length === 0) return resolve(null);
                        
                        const user = results[0];
                        resolve({
                            ...user,
                            dias_pedido: user.dias_pedido ? user.dias_pedido.match(/Segunda|Terça|Quarta|Quinta|Sexta/g) || [] : []
                        });
                    }
                );
            });

            if (!user) return resolve(null);

            const quantidade_minima = await new Promise((resolve, reject) => {
                this.db.query(
                    `SELECT linha, quantidade_minima FROM quantidade_minima WHERE id_user = ?`,
                    [id_user],
                    (error, results) => {
                        if (error) return reject(new Error(error));
                        resolve(results.map(q => ({ linha: q.linha, minimo: q.quantidade_minima})));
                    }
                );
            });

            const lembrete = await new Promise((resolve, reject) => {
              this.db.query(`
                  SELECT mensagens_vinculadas.id_mensagem, mensagens_vinculadas.horario FROM mensagens, mensagens_vinculadas, user
                  WHERE
                  mensagens_vinculadas.id_mensagem = mensagens.id AND
                  mensagens_vinculadas.id_user = user.id AND
                  user.id = ?`,
                  [id_user],
                  (error, results) => {
                      if (error) return reject(new Error(error));
                      resolve(results.map(q => ({ id_mensagem: q.id_mensagem, horario: q.horario })));
                  }
              );
          });

            user.quantidade = quantidade_minima;
            user.lembrete = lembrete;
            return resolve([user]);
        } catch (error) {
            console.error(error);
            return reject(new Error(error));
        }
    });
}  

  listPedidosPorEndereco(endereco) {
    return new Promise(async (resolve, reject) => {
  
  
      try {
        this.db.query(
          `SELECT pedidos.*, user.endereco
          FROM pedidos, user
          WHERE
          pedidos.id_user = user.id AND
          pedidos.concluido = true AND
          user.endereco like ?`,[endereco],
          (error, pedidos) => {
            if (error) return reject(new Error(error));
            return resolve(pedidos);
          }
        );
      } catch (error) {
        console.error(error);
        return reject(new Error(error));
      }
    });
  }

/**
* Lista todos os usuarios que nao efetuaram a compra ontem
* @returns lista com todos os dados de um usuario que nao efeturam compra
*/
listPedidosNaoEfetuados() {
  return new Promise(async (resolve, reject) => {


    try {
      this.db.query(
        `SELECT u.*
        FROM user u
        LEFT JOIN pedidos p ON u.id = p.id_user AND DATE(p.data) = CURDATE() - INTERVAL 1 DAY
        WHERE
        p.id_user IS NULL AND
        u.status like 'aprovado' AND
        u.role != 'ADMIN'`,[],
        (error, usuarios) => {
          if (error) return reject(new Error(error));
          return resolve(usuarios);
        }
      );
    } catch (error) {
      console.error(error);
      return reject(new Error(error));
    }
  });
}


  /**
   * Lista os dados de usuarios aprovados
   * @returns lista com todos os dados de um usuario
   */
  listUserAprovado(data_criado) {
    return new Promise(async (resolve, reject) => {
      const query = data_criado ? `user.data_criado = ? AND` : ``


      try {
        this.db.query(
          `SELECT user.id, user.razao_social, user.documento, user.celular, user.email, user.role, user.status, user.tipo, user.endereco, 
          user.qtde_minima_suino, user.qtde_minima_bovino, user.id_vendedor, user.anotacoes, DATE_FORMAT(user.data_criado, "%d/%m/%Y") AS data_criado,
          (SELECT nome AS nome_vendedor FROM vendedores WHERE vendedores.id = user.id_vendedor) AS nome_vendedor
          FROM user 
          WHERE 
          ${query}
          user.role != "ADMIN" AND
          user.status = 'aprovado'
          ORDER BY user.razao_social`,
          [data_criado ? data_criado : ``],

          (error, usuarios) => {
            if (error) return reject(new Error(error));
            return resolve(usuarios);
          }
        );
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }

  /**
* Lista os dados de usuarios recusados
* @returns lista com todos os dados de um usuario
*/
listUserRecusado(data_criado) {
    return new Promise(async (resolve, reject) => {
      const query = data_criado ? `user.data_criado = ? AND` : ``


      try {
        this.db.query(
          `SELECT user.id, user.razao_social, user.documento, user.celular, user.email, user.role, user.status, user.tipo, user.endereco, 
          user.qtde_minima_suino, user.qtde_minima_bovino, user.id_vendedor, user.anotacoes, DATE_FORMAT(user.data_criado, "%d/%m/%Y") AS data_criado,
          (SELECT nome AS nome_vendedor FROM vendedores WHERE vendedores.id = user.id_vendedor) AS nome_vendedor
          FROM user 
          WHERE 
          ${query}
          user.role != "ADMIN" AND 
          user.status like 'recusado'
          ORDER BY user.razao_social`,
          [data_criado ? data_criado : ""],

          (error, usuarios) => {
            if (error) return reject(new Error(error));
            return resolve(usuarios);
          }
        );
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }

   /**
* Lista os dados de usuarios recusados
* @returns lista com todos os dados de um usuario
*/
listUserRecusados(data_criado) {
  return new Promise(async (resolve, reject) => {
    const query = data_criado ? `user.data_criado = ? AND` : ``


    try {
      this.db.query(
        `SELECT user.id, user.razao_social, user.documento, user.celular, user.email, user.role, user.status, user.tipo, user.endereco, 
        user.qtde_minima_suino, user.qtde_minima_bovino, user.id_vendedor, user.anotacoes, DATE_FORMAT(user.data_criado, "%d/%m/%Y") AS data_criado,
        (SELECT nome AS nome_vendedor FROM vendedores WHERE vendedores.id = user.id_vendedor) AS nome_vendedor
        FROM user 
        WHERE 
        user.role != "ADMIN" AND 
        user.status = 'recusado'
        ORDER BY user.razao_social`,
        [data_criado ? data_criado : ""],

        (error, usuarios) => {
          if (error) return reject(new Error(error));
          return resolve(usuarios);
        }
      );
    } catch (error) {
      return reject(new Error(error));
    }
  });
}

  /**
* Lista os dados de usuarios recusados
* @returns lista com todos os dados de um usuario
*/
  listUserRecusados(id) {
    return new Promise(async (resolve, reject) => {


      try {
        this.db.query(
          `SELECT user.id, user.razao_social, user.documento, user.celular, user.email, user.role, user.status, user.tipo, user.endereco, 
        user.qtde_minima_suino, user.qtde_minima_bovino, user.id_vendedor,
        (SELECT nome AS nome_vendedor FROM vendedores WHERE vendedores.id = user.id_vendedor) AS nome_vendedor
        FROM user 
        WHERE 
        user.role != "ADMIN" AND 
        user.status = 'recusado'`,
          [id ? id : ""],

          (error, usuarios) => {
            if (error) return reject(new Error(error));
            return resolve(usuarios);
          }
        );
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }

  /**
   * Lista os dados usuarios pendente
   * @returns lista com todos os dados de um usuario
   */
  listUserPendente(data_criado) {
    return new Promise(async (resolve, reject) => {
      const query = data_criado ? `user.data_criado = ? AND` : ``


      try {
        this.db.query(
          `SELECT user.id, user.razao_social, user.documento, user.celular, user.email, user.role, user.status, user.tipo, user.endereco, 
          user.qtde_minima_suino, user.qtde_minima_bovino, user.id_vendedor, user.anotacoes, DATE_FORMAT(user.data_criado, "%d/%m/%Y") AS data_criado
          FROM user 
          WHERE 
          ${query}
          user.role != "ADMIN" AND 
          user.status = 'pendente'
          ORDER BY user.razao_social`,
          [data_criado ? data_criado : ""],

          (error, usuarios) => {
            if (error) return reject(new Error(error));
            return resolve(usuarios);
          }
        );
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }


  /**
   * Lista os dados de usuarios aprovados
   * @returns lista com todos os dados de um usuario
   */
  listUserAprovadoTopAS(id) {
    return new Promise(async (resolve, reject) => {


      try {
        this.db.query(
          `SELECT id, razao_social, documento, celular, email, role, status, tipo, endereco, qtde_minima_suino, qtde_minima_bovino  FROM user WHERE user.status = 'aprovado' AND user.tipo = 'TopAS'`,
          [id ? id : ""],

          (error, usuarios) => {
            if (error) return reject(new Error(error));
            return resolve(usuarios);
          }
        );
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }

  /**
* Lista os dados de usuarios aprovados
* @returns lista com todos os dados de um usuario
*/
  listUserAprovadoAtacado(id) {
    return new Promise(async (resolve, reject) => {


      try {
        this.db.query(
          `SELECT id, razao_social, documento, celular, email, role, status, tipo, endereco, qtde_minima_suino, qtde_minima_bovino  FROM user WHERE user.status = 'aprovado' AND user.tipo = 'Atacado'`,
          [id ? id : ""],

          (error, usuarios) => {
            if (error) return reject(new Error(error));
            return resolve(usuarios);
          }
        );
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }

  /**
* Lista pedido que o usuario concluído
* @returns lista com todos os dados de pedidos conluido 
*/
  listUserPedidosConluido(data_concluido) {
    return new Promise(async (resolve, reject) => {
      const query = data_concluido ? `pedidos.data = ?` : `pedidos.data <= ?`
      const data_formatada = data_concluido ? data_concluido : new Date();


      try {
        this.db.query(`SELECT pedidos.id, pedidos.id_user, user.razao_social, user.documento, SUM(pedidos.quantidade) AS quantidade_total, DATE_FORMAT(pedidos.data, '%d/%m/%Y') AS data, pedidos.hora, user.aroma
                  FROM user, pedidos
                  WHERE
                  pedidos.id_user = user.id AND
                  pedidos.concluido = true AND
                  ${query}
                  GROUP BY user.id, pedidos.data
                  ORDER BY pedidos.data DESC, pedidos.hora DESC `,
          [data_formatada],

          (error, usuarios) => {
            if (error) return reject(new Error(error));
            return resolve(usuarios);
          }
        );
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }

  /**
* Lista com a quantidade de usuarios registrados 
* @returns lista com todos os dados de um usuario
*/
  countAllUserTypes() {
    return new Promise(async (resolve, reject) => {


      try {
        this.db.query(`SELECT  
                  (
                    SELECT COUNT(*) FROM vendedores
                  ) AS vendedores,
                  (
                    SELECT COUNT(*) FROM user WHERE user.status like "aprovado"
                  ) AS aprovados,
                  (
                    SELECT COUNT(*) FROM user WHERE user.status like "recusado"
                  ) AS recusados,
                  (
                    SELECT COUNT(*) FROM user WHERE user.status like "pendente"
                  ) AS pendentes
                  FROM 
                  user
                  WHERE 
                  user.role != 'ADMIN'
                  GROUP BY vendedores`, [],

          (error, usuarios) => {
            if (error) return reject(new Error(error));
            return resolve(usuarios);
          }
        );
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }

  /**
   * retorna uma lista de usuários e seus pedidos em uma data específica. 
   * Ela categoriza os usuários em três status diferentes: feito, antecipado e pendente, com base nos pedidos realizados e nos dias de pedido registrados.
   * @param {String} data YYYY-MM-DD
   * @returns retorna todos usuarios que fizeram pedido hoje ou dexaram de fazer
   */
  listAllUserDiaPedido(data) {
    return new Promise(async (resolve, reject) => {
      
      try {
        const pedidosHoje = await new Promise((resolve, reject) => {
          this.db.query(
            `SELECT user.id, user.razao_social, user.dias_pedido, pedidos.data, user.celular
            FROM user, pedidos
            WHERE 
            pedidos.id_user = user.id AND
            pedidos.concluido = true AND
            DATE(pedidos.data) = ?`,
            [data],
            (error, results) => {
              if (error) {
                console.error("Error fetching existing order:", error);
                return reject(error);
              }
              resolve(
                results.map(usuario => ({
                  ...usuario,
                  dias_pedido: usuario.dias_pedido ? usuario.dias_pedido.match(/Segunda|Terça|Quarta|Quinta|Sexta/g) || [] : []
                }))
              );
            }
          );
        });
        const userPedidosHoje = await new Promise((resolve, reject) => {
          this.db.query(
            `SELECT u.id, u.razao_social, u.dias_pedido, u.celular
            FROM user u
            LEFT JOIN pedidos p ON u.id = p.id_user AND DATE(p.data) = CURDATE()
            WHERE
            p.id_user IS NULL AND
            u.status LIKE 'aprovado' AND
            u.role != 'ADMIN' AND 
            u.dias_pedido LIKE CONCAT('%', 
            CASE DAYOFWEEK(?)
                WHEN 1 THEN 'Domingo'
                WHEN 2 THEN 'Segunda'
                WHEN 3 THEN 'Terça'
                WHEN 4 THEN 'Quarta'
                WHEN 5 THEN 'Quinta'
                WHEN 6 THEN 'Sexta'
            END, 
            '%')`,
            [data],
            (error, results) => {
              if (error) {
                console.error("Error fetching non-ordered users:", error);
                return reject(error);
              }
              resolve(
                results.map(usuario => ({
                  ...usuario,
                  dias_pedido: usuario.dias_pedido ? usuario.dias_pedido.match(/Segunda|Terça|Quarta|Quinta|Sexta/g) || [] : []
                }))
              );
            }
          );
        });

        const daysOfWeek = ['Domingo', 'Segunda-feira', 'Terça-feira', 'Quarta-feira', 'Quinta-feira', 'Sexta-feira', 'Sábado'];

        const date = new Date(`${data}T00:00:00`); 
        

      
        const usuariosMap = new Map();
    
        pedidosHoje.forEach(usuario => {
          usuariosMap.set(usuario.id, {
            ...usuario,
            status: usuario.dias_pedido.includes(daysOfWeek[date.getDay()].replace('-feira', '')) ? 'Feito' : 'Fora de rota'
          });
        });
    
        userPedidosHoje.forEach(usuario => {
          if (!usuariosMap.has(usuario.id)) {
            usuariosMap.set(usuario.id, {
              ...usuario,
              status: 'Pendente'
            });
          }
        });
    
        return resolve(Array.from(usuariosMap.values()));
      } catch (error) {
        reject(error);
      }
    });
       
  }
  
  


  /**
* Lista os dados de usuarios aprovados
* @returns lista com todos os dados de um usuario
*/
  listUserAprovadoDistribuidor(id) {
    return new Promise(async (resolve, reject) => {


      try {
        this.db.query(
          `SELECT id, razao_social, documento, celular, email, role, status, tipo, endereco, qtde_minima_suino, qtde_minima_bovino  FROM user WHERE user.status = 'aprovado' AND user.tipo = 'Distribuidor'`,
          [id ? id : ""],

          (error, usuarios) => {
            if (error) return reject(new Error(error));
            return resolve(usuarios);
          }
        );
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }

  listTopVendedores() {
    return new Promise(async (resolve, reject) => {


      try {
        this.db.query(
          `SELECT 
            vendedores.id,
            vendedores.nome,
            COALESCE(SUM(CASE 
              WHEN pedidos.concluido = true
              THEN pedidos.quantidade 
            END), 0) AS quantidade_vendida
            FROM pedidos
            JOIN user ON pedidos.id_user = user.id
            JOIN produtos ON pedidos.id_produto = produtos.id
            JOIN vendedores ON user.id_vendedor = vendedores.id
            WHERE pedidos.concluido = TRUE
            GROUP BY vendedores.nome
            ORDER BY quantidade_vendida DESC
            LIMIT 5;`,
          [],(error, usuarios) => {
            if (error) return reject(new Error(error));
            return resolve(usuarios);
          });
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }


  listTopCidades() {
    return new Promise(async (resolve, reject) => {


      try {
        this.db.query(
          `SELECT 
        user.endereco,
        COALESCE(SUM(CASE 
          WHEN pedidos.concluido = true
          THEN pedidos.quantidade 
        END), 0) AS quantidade_vendida
          FROM pedidos
          JOIN user ON pedidos.id_user = user.id
          JOIN produtos ON pedidos.id_produto = produtos.id
          JOIN vendedores ON user.id_vendedor = vendedores.id
          
          WHERE 
            pedidos.concluido = TRUE AND
            user.endereco IS NOT NULL
          GROUP BY user.endereco
          ORDER BY quantidade_vendida DESC
`,
          [],(error, usuarios) => {
            if (error) return reject(new Error(error));
            return resolve(usuarios);
          });
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }

  /**
  * Lista todos os pedidos concluido de um usuario 
  * @returns lista com todos os pedidos concluidos de um usuario
  */
  listHistoricoPedidoByUser(id_user, data_inicio, data_fim) {
    return new Promise(async (resolve, reject) => {


      try {
        let da = this.db.query(
          `SELECT pedidos.id, pedidos.id_user, pedidos.id_produto, pedidos.quantidade,  pedidos.concluido,  DATE_FORMAT(pedidos.data, "%d/%m/%Y") AS data, 
          pedidos.hora,produtos.nome, produtos.image,  produtos.descricao
          FROM 
              user, pedidos, produtos
          WHERE 
              pedidos.id_user = user.id 
              AND pedidos.id_produto = produtos.id 
              AND pedidos.concluido = true 
              AND user.id = ?
              AND pedidos.data BETWEEN ? AND ?
          ORDER BY pedidos.data DESC, pedidos.hora DESC;
`,
          [id_user, data_inicio, data_fim],
          (error, usuarios) => {
            if (error) return reject(new Error(error));
        
            return resolve(usuarios);
          }
        );
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }




  /**
* Lista os dados de usuarios exetos os admins
* @returns lista com todos os dados de um usuario
*/
listAllUser(id) {
  return new Promise(async (resolve, reject) => {
    try {
      const users = await new Promise((resolve, reject) => {
        this.db.query(
          `SELECT user.id, user.razao_social, user.documento, user.celular, user.email, user.role, user.status, user.tipo, user.aroma,
                user.dias_pedido, user.endereco, user.qtde_minima_suino, user.qtde_minima_bovino, user.qtde_minima_cross, user.linha, user.id_vendedor, user.hora_lembrete,
                user.anotacoes, DATE_FORMAT(user.data_criado, "%d/%m/%Y") AS data_criado, DATE_FORMAT(pedidos.data, "%d/%m/%Y") AS data_pedido, SUM(pedidos.quantidade) AS quantidade_total_pedidos
          FROM user
          LEFT JOIN pedidos ON pedidos.id_user = user.id
          WHERE user.role != 'ADMIN'
          GROUP BY user.id
          ORDER BY FIELD(user.status, 'pendente', 'aprovado', 'recusado'), user.data_criado DESC, user.razao_social`,
          [id ? id : ""],
          (error, usuarios) => {
            if (error) return reject(new Error(error));

            const usuariosTransformados = usuarios.map(usuario => ({
              ...usuario,
              dias_pedido: usuario.dias_pedido ? usuario.dias_pedido.match(/Segunda|Terça|Quarta|Quinta|Sexta/g) || [] : [],
              quantidade_minima: [] // Inicializa a propriedade para cada usuário
            }));
            resolve(usuariosTransformados);
          }
        );
      });

      for (let user of users) {
        const qtdeMinima = await new Promise((resolve, reject) => {
          this.db.query(
            `SELECT * FROM quantidade_minima WHERE id_user = ?`,
            [user.id],
            (error, results) => {
              if (error) {
                console.error("Error fetching quantidade_minima:", error);
                return reject(error);
              }
              resolve(results);
            }
          );
        });
        user.quantidade_minima = qtdeMinima;
      }

      return resolve(users);
    } catch (error) {
      return reject(new Error(error));
    }
  });
}

  /**
  * Lista os dados de usuarios exetos os admins
  * @returns lista com todos os dados de um usuario
  */
  listUserHistorico(id) {
    return new Promise(async (resolve, reject) => {
      try {
        this.db.query(`SELECT 
          user.id AS id_user, 
          user.celular AS celular_user,
          user.razao_social, 
          user.documento,  
          user.meta_mensal,
          user.dias_pedido,
          (
            SELECT DATE_FORMAT(MAX(pedidos.data), "%d/%m/%Y")
            FROM pedidos
            WHERE pedidos.id_user = user.id
          ) AS data_pedido,
          (
            SELECT nome
            FROM vendedores
            WHERE user.id_vendedor = vendedores.id 
          ) AS nome_vendedor,
          (
            SELECT celular
            FROM vendedores
            WHERE user.id_vendedor = vendedores.id 
          ) AS celular_vendedor,
          (
              select SUM(pedidos.quantidade)
              from pedidos
              WHERE 
               pedidos.data >= DATE_SUB(CURDATE(), INTERVAL WEEKDAY(CURDATE()) + 7 DAY)
				        AND pedidos.data < DATE_SUB(CURDATE(), INTERVAL WEEKDAY(CURDATE()) DAY) 
                AND pedidos.concluido = true AND pedidos.id_user = user.id
          ) AS quantidade_ultima_semana,
          (
            SELECT SUM(pedidos.quantidade)
            FROM pedidos
            WHERE YEARWEEK(pedidos.data, 1) = YEARWEEK(CURDATE(), 1) AND pedidos.concluido = true AND pedidos.id_user = user.id
          ) AS quantidade_semana_atual,
          (
            SELECT SUM(pedidos.quantidade) AS quantidade 
            FROM pedidos 
            WHERE
            pedidos.concluido = true
            AND YEAR(pedidos.data) = YEAR(CURDATE() - INTERVAL 1 MONTH) 
            AND MONTH(pedidos.data) = MONTH(CURDATE() - INTERVAL 1 MONTH)
            AND pedidos.id_user = user.id
          ) AS quantidade_total
        FROM user
        LEFT JOIN pedidos ON pedidos.id_user = user.id
        WHERE 
          user.role != 'ADMIN' AND
          user.status LIKE 'aprovado'
        GROUP BY user.id
        ORDER BY  pedidos.data ASC`, 
          [id ? id : ""],
          (error, usuarios) => {
            if (error) return reject(new Error(error));
                
            const historicoUser = [];
            let usuarioAtual = null;
            let dataPedidos = [];



            for (let x = 0; x < usuarios.length; x++) {
              const usuario = usuarios[x];

              // Novo usuário encontrado ou usuário sem pedidos
              if (!usuarioAtual || usuario.id_user !== usuarioAtual.id_user) {
                if (usuarioAtual) {
                  usuarioAtual.data_pedidos = [...new Set(dataPedidos)];
                  historicoUser.push(usuarioAtual);
                }

                // Inicializa novo usuário
                usuarioAtual = {
                  id_user: usuario.id_user,
                  razao_social: usuario.razao_social,
                  celular_user: usuario.celular_user,
                  documento: usuario.documento,
                  meta_mensal: usuario.meta_mensal,
                  dias_pedido: usuario.dias_pedido ? usuario.dias_pedido.match(/Segunda|Terça|Quarta|Quinta|Sexta/g) || [] : [],
                  data_ultimo_pedido: usuario.data_pedido, // Armazena a data mais recente
                  quantidade_ultima_semana: usuario.quantidade_ultima_semana,
                  quantidade_semana_atual: usuario.quantidade_semana_atual,
                  quantidade_total: usuario.quantidade_total,
                  nome_vendedor: usuario.nome_vendedor,
                  celular_vendedor: usuario.celular_vendedor
                };
                dataPedidos = usuario.data_pedido ? [usuario.data_pedido] : []; // Inicia array de pedidos
              } else if (usuario.data_pedido) {
                // Adiciona outras datas de pedidos ao array
                dataPedidos.push(usuario.data_pedido);
              }
            }

            // Adiciona o último usuário processado
            if (usuarioAtual) {
              usuarioAtual.data_pedidos = [...new Set(dataPedidos)];
              historicoUser.push(usuarioAtual);
            }


            return resolve(historicoUser);
        
          }
        );
      } catch (error) {
        console.error(error);
        return reject(new Error(error));
      }
    });
  }



  /**
   * Deleta o usuario cadastrado
   * @param {*} id ID do usuario a ser excluído
   * @returns mensagem referente ao resultado obtido
   */
  delete(id) {
    return new Promise((resolve, reject) => {
      this.db.query(
        "DELETE FROM user WHERE id = ?",
        [id],
        (error, response) => {
          if (error) {
            return resolve({
              error: "Este usuarios possui vinculos!",
              code: 203,
            });
          } else {
            return resolve(true);
          }
        }
      );
    });
  }

  /**
   * Atualiza os dados de um usuario
   * @param {*} data dados atualizados do usuario
   * @returns mensagem referente ao resultado obtido
   */
  update(data) {
    return new Promise(async (resolve, reject) => {
      this.db.query(
        "UPDATE usuarios SET nome=?, email=?, perfil=? WHERE id=?",
        [data.nome, data.email, data.perfil, data.id],
        async (error, response) => {
          if (error) return reject(new Error(error));
          else return resolve({ success: "Usuario atualizado com sucesso!" });
        }
      );
    });
  }



  /**
   * Desvincula um vendedor de um user
   * @param {*} data dados do vendedor vinculado a um user
   * @returns mensagem referente ao resultado obtido
   */
  desvincularVendedor(data) {
    return new Promise(async (resolve, reject) => {
      this.db.query(
        "UPDATE user SET id_vendedor=? WHERE id=?",
        [null, data.id],
        async (error, response) => {
          if (error) {
            console.error(error);

            return resolve({ error: "Erro ao desvincular vendedor de usuario", code: 500 })
          }
          else return resolve({ success: "Vendedor desvinculado com sucesso!", code: 200 });
        }
      );
    });
  }


  /**
 * Atualiza todos os dados de um usuarios
 * @param {*} data dados atualizados do usuario
 * @returns mensagem referente ao resultado obtido
 */
  updateFullUser(data) {
    return new Promise(async (resolve, reject) => {
      this.db.query(
        `UPDATE user SET razao_social=?, email=?, documento=?, celular=?, role=?,
           status=?, tipo=?, endereco=?, qtde_minima_suino=?, qtde_minima_bovino=?,
           id_vendedor=? WHERE id=?`,
        [data.razao_social, data.email, data.documento, data.celular, data.role, data.status,
        data.tipo, data.endereco, data.qtde_minima_suino,
        data.qtde_minima_bovino, data.id_vendedor, data.id],
        async (error, response) => {
          if (error) return reject(new Error(error));
          else return resolve({ success: "Usuario atualizado com sucesso!" });
        }
      );
    });
  }

  /**
   * Criptografa um valor
   * @param {*} data valor recebido
   * @returns o valor criptografado
   */
  encrypt(data) {
    return new Promise(async (resolve, reject) => {
      try {
        let value;
        if (data.hash) {
          value = await generateHash(data.value);
        } else {
          value = await encrypt(data.value);
        }
        return resolve({ value: value });
      } catch (error) {
        return resolve({ error: `Não foi possível criptografar!`, code: 203 });
      }
    });
  }

    
  updateUltra(data) {
    return new Promise((resolve, reject) => {
      const fields = [];
      const values = [];
  
      if (data.ultra !== undefined) {
        fields.push('ultra = ?');
        values.push(data.ultra);
      }
  
      if (data.faturamento !== undefined) {
        fields.push('faturamento = ?');
        values.push(data.faturamento);
      }
  
      if (fields.length === 0) {
        return resolve({ error: "Nenhum dado para atualizar", code: 400 });
      }

      console.log('fields', fields)
  
      const query = `UPDATE ultra SET ${fields.join(', ')} WHERE id = 1`;
  
      this.db.query(query, values, (error, response) => {
        if (error) {
          console.error(error);
          return resolve({ error: "Este pedido possui vínculos!", code: 203 });
        }
  
        return resolve({ success: "Dados atualizados com sucesso!", code: 200 });
      });
    });
  }
  


  listUltra(data) {
    return new Promise(async (resolve, reject) => {
      this.db.query(`SELECT * FROM ultra`,[],
          async (error, ultra) => {
              if (error) {
                  console.error(error);
                  return reject(new Error(error));
              }

              return resolve(ultra[0]);
          })
    });
  }

}

module.exports = usersRepository;
